! Molecular Orbital PACkage (MOPAC)
! Copyright (C) 2021, Virginia Polytechnic Institute and State University
!
! MOPAC is free software: you can redistribute it and/or modify it under
! the terms of the GNU Lesser General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! (at your option) any later version.
!
! MOPAC is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU Lesser General Public License for more details.
!
! You should have received a copy of the GNU Lesser General Public License
! along with this program.  If not, see <https://www.gnu.org/licenses/>.

subroutine mopac_gui_activate
!dec$ attributes dllexport :: mopac_gui_activate
  use molkst_C, only : gui
  implicit none
  gui = .true.
  return
end subroutine mopac_gui_activate

subroutine mopac_gui_deactivate
!dec$ attributes dllexport :: mopac_gui_deactivate
  use molkst_C, only : gui
  implicit none
  gui = .false.
  return
end subroutine mopac_gui_deactivate

subroutine mopac_gui_set_channel(channel)
!dec$ attributes dllexport :: mopac_gui_set_channel
  use chanel_C, only : iw0
  implicit none
  integer :: channel
  iw0 = channel
  return
end subroutine mopac_gui_set_channel

subroutine mopac_gui_get_channel(channel)
!dec$ attributes dllexport :: mopac_gui_get_channel
  use chanel_C, only : iw0
  implicit none
  integer :: channel
  channel = iw0
  return
end subroutine mopac_gui_get_channel
