mopac ag20.mop
rm ag20.arc

mopac ag20_cis_active.mop

mopac benzene.mop

mopac co_cas.mop

mopac co_cis.mop

mopac co_cis_2.mop

mopac co_cis_PM7.mop
rm co_cis_PM7.arc

mopac co_cis_active.mop

mopac co_cis_solv.mop

mopac co_cis_tdip.mop
rm co_cis_tdip.tdip

mopac co_cis_trip.mop

mopac co_extern.mop

mopac co_fail.mop

mopac co_field.mop

mopac co_mrci.mop

mopac co_mrci_maxci.mop

mopac co_mrci_solv.mop

mopac co_pm7.mop
rm co_pm7.arc

mopac co_pm7_solv.mop
rm co_pm7_solv.arc

mopac co_print.mop

mopac no_cis_quar.mop

mopac no_mrci.mop

mopac noparam.mop

mopac poly_c8.mop

mopac poly_c8_mr.mop
